
local Class = require"libs.middleclass_320"
local Stateful = require"libs.stateful_103"
local UI = require"ui"
local Spielfeld = require"spielfeld"
local Stack = require"stack"

--global because states in different files ( https://love2d.org/forums/viewtopic.php?f=5&t=3756#p44011 ):
Game = Class("Game"):include(Stateful)

function Game:initialize()
	
	self.fontSet = { }
	self.fontSet[1] = {
		love.graphics.newFont("ress/nova-mono_400.ttf", 10),
		love.graphics.newFont("ress/nova-mono_400.ttf", 14),
		love.graphics.newFont("ress/nova-mono_400.ttf", 18),
		love.graphics.newFont("ress/nova-mono_400.ttf", 24),
		love.graphics.newFont("ress/nova-mono_400.ttf", 48),
		love.graphics.newFont("ress/nova-mono_400.ttf", 72)
	}
	self.fontSet[2] = {
		love.graphics.newFont("ress/nova-mono_400.ttf", 12),
		love.graphics.newFont("ress/nova-mono_400.ttf", 18),
		love.graphics.newFont("ress/nova-mono_400.ttf", 24),
		love.graphics.newFont("ress/nova-mono_400.ttf", 36),
		love.graphics.newFont("ress/nova-mono_400.ttf", 72),
		love.graphics.newFont("ress/nova-mono_400.ttf", 128)
	}
	self.font = self.fontSet[1]
	
	self.winX, self.winY = love.graphics.getDimensions()
	self.ui = UI{font = self.font, winX = self.winX, winY = self.winY, isFS = self.isFS}
	self.isFS = false -- is fullscreen?
	
	self.displayStats = { }
	self.displayStats.score = 0
	self.displayStats.turns = 1
	
	self.playermove_start = nil
	
	self:gotoState("Menu")
	
end



function Game:draw()
end

function Game:keypressed(key)
end

function Game:mousepressed(x, y, mb)
end

function Game:mousemoved(x, y, dx, dy)
end



function Game:checkWholeMap(coordStr)
	
	local dirs = { {0, 1}, {1, 0}, {1, -1}, {1, 1} }
	local toDelete = { }
	local gotHits = false
	for x = 1, self.ui.map.size do
		for y = 1, self.ui.map.size do
			
			local spot = Spielfeld.static.getCoordString(x, y)
			for i, v in ipairs(dirs) do
				local result = self:checkBeam(spot, v[1], v[2])
				if result then
					for ii, vv in ipairs(result) do
						toDelete[vv] = true
						gotHits = true
					end
				end
			end
			
		end
	end
	
	if gotHits then
		toDelete[coordStr] = true
		for i, v in pairs(toDelete) do
			local special = self.ui.map.map[i].special
			local bonus = 1
			if special == "locked" then bonus = 6 end
			if special == "bomb" then self:handleSpecial(special, i) end
			self.ui.map:setPos{coordStr = i, size = 0, col = 1, special = "none"}
			self.displayStats.score = self.displayStats.score + bonus
		end
	end
	
end



function Game:checkForFive(coordStr)
	
	local dirs = { {0, 1}, {1, 0}, {1, -1}, {1, 1} }
	local toDelete = { }
	
	for i, v in ipairs(dirs) do
		local result = self:checkBeam(coordStr, v[1], v[2])
		if result then
			for ii, vv in ipairs(result) do
				table.insert(toDelete, vv)
			end
		end
	end
	
	if #toDelete > 0 then
		table.insert(toDelete, coordStr)
		for i, v in ipairs(toDelete) do
			local special = self.ui.map.map[v].special
			local bonus = 1
			if special == "locked" then bonus = 6 end
			if special and special ~= "none" then self:handleSpecial(special, v) end
			self.ui.map:setPos{coordStr = v, size = 0, col = 1, special = "none"}
			self.displayStats.score = self.displayStats.score + bonus
		end
	end
	
end



function Game:checkBeam(coordStr, rx, ry)
	
	local cx, cy = Spielfeld.static.splitCoordString(coordStr)
	local col = self.ui.map.map[coordStr].col
	local dx, dy, examCol, examSize, examCoordStr = cx, cy, col, 2, nil
	local retTbl = { }
	
	while (examCol == col) and (examSize == 2) do
		if examCoordStr and examCoordStr ~= coordStr then table.insert(retTbl, examCoordStr) end
		dx = dx + rx
		dy = dy + ry
		examCoordStr = Spielfeld.static.getCoordString(dx, dy)
		examCol = self.ui.map.map[examCoordStr].col
		examSize = self.ui.map.map[examCoordStr].size
	end
	
	dx, dy, examCol, examSize, examCoordStr = cx, cy, col, 2, nil
	while (examCol == col) and (examSize == 2) do
		if examCoordStr and examCoordStr ~= coordStr then table.insert(retTbl, examCoordStr) end
		dx = dx - rx
		dy = dy - ry
		examCoordStr = Spielfeld.static.getCoordString(dx, dy)
		examCol = self.ui.map.map[examCoordStr].col
		examSize = self.ui.map.map[examCoordStr].size
	end
	
	if #retTbl >= 4 then return retTbl end -- 4 is enough, because not counting startpos (otherwise might cause overlap later)
	
end



function Game:handleSpecial(special, coordStr)
	
	local x, y = Spielfeld.static.splitCoordString(coordStr)
	
	if special == "bomb" then
		for dx = -1, 1 do
			for dy = -1, 1 do
				local newX, newY = x + dx, y + dy
				if newX >= 1 and newY >= 1 and newX <= self.ui.numOfFields and newY <= self.ui.numOfFields then
					self.ui.map:setPos{x = x + dx, y = y + dy, col = 1, size = 0, special = "none"}
					self.displayStats.score = self.displayStats.score + 1
				end
			end
		end
		for i, v in ipairs(self.ui.map.mapByContent[1]) do
			self.ui.map:setPos{coordStr = v, size = 0, col = 1, special = "none"}
		end
		
	elseif special == "recolor" then
		local c1 = self.ui.map.map[coordStr].col
		local c2 = love.math.random(#Spielfeld.static.colors)
		while c1 == c2 do
			c2 = love.math.random(#Spielfeld.static.colors)
		end
		for ix = 1, self.ui.numOfFields do
			for iy = 1, self.ui.numOfFields do
				local iCoor = Spielfeld.static.getCoordString(ix, iy)
				if self.ui.map.map[iCoor].size > 0 and self.ui.map.map[iCoor].col == c1 then
					self.ui.map.map[iCoor].col = c2
				end
			end
		end
		self:checkForFive(coordStr)
		self:checkWholeMap(coordStr)
		
	end
	
end



function Game:undo1stClick()
	
	self.ui.map:clearFFcol()
	self.floodfillColorindex = 0
	self.ui.map.map[self.playermove_start].highl = false
	self.playermove_start = nil
	
end



function Game:toggleFullscreen()
	
	self.isFS = love.window.getFullscreen()
	
	if self.isFS then
		self.font = self.fontSet[1]
			love.window.setFullscreen(false)
	else
		self.font = self.fontSet[2]
		love.window.setFullscreen(true)
	end
	
	self.isFS = not self.isFS
	self.winX, self.winY = love.graphics.getDimensions()
	self.ui.font = self.font
	self.ui.winX = self.winX
	self.ui.winY = self.winY
	self.ui.isFS = self.isFS
	
	local maxMapSizePx = math.min(self.ui.winX * 0.67, self.ui.winY*0.8)
	self.ui.mapY_Offs = (self.ui.winY - maxMapSizePx) / 2
	self.ui.mapX_Offs = self.ui.mapY_Offs
	self.ui.fieldSizePx = math.floor(maxMapSizePx / self.ui.map.size)
	
	self.ui.sideBoxX = maxMapSizePx + self.ui.mapX_Offs * 2
	self.ui.sideBoxY = self.ui.mapY_Offs
	self.ui.sideBoxW = self.ui.winX - self.ui.sideBoxX - self.ui.mapX_Offs
	self.ui.sideBoxH = self.ui.winY - self.ui.mapY_Offs * 2
	
	self.ui.map.mapSizePx = self.ui.map.size * self.ui.fieldSizePx
	self.ui.map.fieldSizePx = self.ui.fieldSizePx
	
	self.ui.map.fieldCanvas = love.graphics.newCanvas(self.ui.map.fieldSizePx, self.ui.map.fieldSizePx)
	self.ui.map.canvas = love.graphics.newCanvas(self.ui.map.fieldSizePx * self.ui.map.size, self.ui.map.fieldSizePx * self.ui.map.size)
	
end







