
local Class = require"libs.middleclass_320"
local Stateful = require"libs.stateful_103"
local Spielfeld = require"spielfeld"

local UI = Class("UI"):include(Stateful)



function UI:initialize(params)
	
	self.winX, self.winY, self.isFS = params.winX, params.winY, params.isFS
	
	local maxMapSizePx = math.min(self.winX * 0.67, self.winY*0.8)
	self.mapY_Offs = (self.winY - maxMapSizePx) / 2
	self.mapX_Offs = self.mapY_Offs
	self.numOfFields = 10
	self.fieldSizePx = math.floor(maxMapSizePx / self.numOfFields)
	
	self.sideBoxX = maxMapSizePx + self.mapX_Offs * 2
	self.sideBoxY = self.mapY_Offs
	self.sideBoxW = self.winX - self.sideBoxX - self.mapX_Offs
	self.sideBoxH = self.winY - self.mapY_Offs * 2
	
	self.font = params.font
	
	self.map = Spielfeld{fieldSizePx = self.fieldSizePx, numOfFields = self.numOfFields}
	
	self.debugInfo = ""
	self.debug_playermove_start = ""

end



function UI:draw(stats)
	
	local y
	
	if self.isFS then y = -40 else y = -25 end
	love.graphics.setFont(self.font[6])
	love.graphics.printf("Fives", self.sideBoxX, self.sideBoxY + y, self.sideBoxW, "center")
	
	if self.isFS then y = 110 else y = 60 end
	love.graphics.setFont(self.font[3])
	love.graphics.printf("a remake by riidom", self.sideBoxX, self.sideBoxY + y, self.sideBoxW, "center")
	
	local score_s, turns_s = "", ""
	if stats.score ~= 1 then score_s = "s" end
	if stats.turns ~= 1 then turns_s = "s" end
	
	love.graphics.setFont(self.font[2])
	local text1 = {
		stats.score.." point"..score_s.." you scored.",
		stats.turns.." turn"..turns_s.." you managed to survive."
	}
	if self.isFS then y = 200 else y = 140 end
	love.graphics.printf(table.concat(text1, "\n"), self.sideBoxX, self.sideBoxY + y, self.sideBoxW, "left")
	
	love.graphics.setFont(self.font[1])
	if self.isFS then y = 400 else y = 200 end
	--love.graphics.printf(self.debugInfo, self.sideBoxX, self.sideBoxY + y, self.sideBoxW, "left")
	love.graphics.print(love.timer.getFPS( ), 0, 0)

end



function UI:getClickedField(x, y)
	
	local fx, fy, coordStr
	if x > self.mapX_Offs and x < self.mapX_Offs + self.map.mapSizePx
	and y > self.mapY_Offs and y < self.mapY_Offs + self.map.mapSizePx then
		local rx, ry = x - self.mapX_Offs, y - self.mapY_Offs
		fx, fy = math.floor(rx / self.fieldSizePx) + 1, math.floor(ry / self.fieldSizePx) + 1
		coordStr = Spielfeld.static.getCoordString(fx, fy)
	end
	return fx, fy, coordStr
	
end



function UI:displayFieldDebugInfo(coordStr, x, y)
	
	local output = { }
	table.insert(output, coordStr)
	table.insert(output, "")
	
	for k, v in pairs(self.map.map[coordStr]) do
		table.insert(output, "\t"..k..": "..tostring(v))
	end
	
	table.insert(output, "")
	table.insert(output, "\tmap.floodfillColorindex: "..self.map.floodfillColorindex)
	table.insert(output, "\tgame.playermove_start: "..self.debug_playermove_start)
	self.debugInfo = table.concat(output, "\n")
	
end


	
return UI



